/**
 * Copyright (c) 2006 - 2008 Smaxe Ltd (www.smaxe.com).
 * All rights reserved.
 */

import com.smaxe.uv.client.camera.MediaStreamCamera;
import com.smaxe.uv.client.microphone.MediaStreamMicrophone;
import com.smaxe.uv.client.rtmp.INetConnection;
import com.smaxe.uv.client.rtmp.INetStream;
import com.smaxe.uv.client.rtmp.MediaDataFactory;
import com.smaxe.uv.stream.IMediaStream;
import com.smaxe.uv.stream.IMediaStreamController;
import com.smaxe.uv.stream.support.MediaStreamController;

import java.io.File;
import java.util.Map;

/**
 * <code>ExRtmpPublishStream</code> - example that shows how to publish local FLV file as live stream.
 * 
 * @author Andrei Sochirca
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-rtmp-client" target="_blank">JUV RTMP Client</a>
 */
public final class ExRtmpPublishStream extends Object
{
    /**
     * Entry point.
     * 
     * @param args
     */
    public static void main(final String[] args)
    {
        // NOTE:
        // you can get Evaluation Key at:
        // http://www.smaxe.com/order.jsf#request_evaluation_key
        // or buy at:
        // http://www.smaxe.com/order.jsf
        
        // Android-specific:
        // - please add permission to the AndroidManifest.xml : 
        // <uses-permission android:name="android.permission.INTERNET" />
        // - please use separate thread to connect to the server (not UI thread) : 
        // NetConnection#connect() connects to the remote server in the invocation thread,
        // so it causes NetworkOnMainThreadException on 4.0 (http://developer.android.com/reference/android/os/NetworkOnMainThreadException.html)
        
        // YouTube-specific:
        // - YouTube requires both audio and video stream
        com.smaxe.uv.client.rtmp.License.setKey("SET-YOUR-KEY");
        
        final ExRtmpPublishStream app = new ExRtmpPublishStream();
        
        // EdgeCast stream: <streamName>?<Live Authentication Key>&param1=value1&param2=value2
        app.start(new com.smaxe.uv.client.rtmp.NetConnection(), "rtmp://localhost:1935/live", "stream");
    }
    
    /**
     * <code>NetConnectionListener</code> - {@link INetConnection} listener implementation.
     */
    public class NetConnectionListener extends INetConnection.ListenerAdapter
    {
        /**
         * Constructor.
         */
        public NetConnectionListener()
        {
        }
        
        @Override
        public void onAsyncError(final INetConnection source, final String message, final Exception e)
        {
            System.out.println("NetConnection#onAsyncError: " + message + " " + e);
        }
        
        @Override
        public void onIOError(final INetConnection source, final String message)
        {
            System.out.println("NetConnection#onIOError: " + message);
        }
        
        @Override
        public void onNetStatus(final INetConnection source, final Map<String, Object> info)
        {
            System.out.println("NetConnection#onNetStatus: " + info);
            
            final Object code = info.get("code");
            
            if (INetConnection.CONNECT_SUCCESS.equals(code))
            {
            }
            else
            {
                disconnected = true;
            }
        }
    }
    
    // fields
    private volatile boolean disconnected = false;
    
    /**
     * Constructor.
     */
    public ExRtmpPublishStream()
    {
    }
    
    /**
     * Starts the example.
     * 
     * @param connection
     * @param url
     * @param streamName
     */
    public void start(final INetConnection connection, final String url, final String streamName)
    {
        connection.addEventListener(new NetConnectionListener());
        
        connection.connect(url);
        
        // wait till connected
        while (!connection.connected() && !disconnected)
        {
            try
            {
                Thread.sleep(100);
            }
            catch (Exception e) {/*ignore*/}
        }
        
        final MediaStreamController controller = new MediaStreamController();
        
        final File streamFile = new File("stream.mp3");
        
        controller.addListener(new IMediaStreamController.ListenerAdapter()
        {
            @Override
            public void onStart(final long timestamp, final IMediaStream stream)
            {
                System.out.println("onStart: " + timestamp + " " + stream);
            }
            
            @Override
            public void onStop(final long timestamp, final IMediaStream stream)
            {
                System.out.println("onStop: " + timestamp + " " + stream);
                
                if (stream != null)
                {
                    stream.release();
                }
                
                try
                {
                    controller.play(MediaDataFactory.createMp3MediaStream(streamFile), -1 /*play the whole file*/);
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
            
            @Override
            public void onRelease(final long timestamp)
            {
                System.out.println("onRelease: " + timestamp);
            }
        });
        
        if (!disconnected)
        {
            final INetStream stream = connection.createNetStream();
            
            stream.addEventListener(new INetStream.ListenerAdapter()
            {
                @Override
                public void onNetStatus(final INetStream source, final Map<String, Object> info)
                {
                    System.out.println("NetStream#onNetStatus: " + info);
                    
                    final Object code = info.get("code");
                    
                    if (INetStream.PUBLISH_START.equals(code))
                    {
                        try
                        {
                            controller.play(MediaDataFactory.createMp3MediaStream(streamFile), -1 /*play the whole file*/);
                        }
                        catch (Exception e)
                        {
                            e.printStackTrace();
                        }
                    }
                }
            });
            
            stream.attachAudio(new MediaStreamMicrophone(controller));
            stream.attachCamera(new MediaStreamCamera(controller), -1);
            
            stream.publish(streamName, INetStream.LIVE);
            
            try
            {
                Thread.sleep(30 * 1000);
            }
            catch (Exception e) {/*ignore*/}
        }
        
        connection.close();
    }
}